import { __awaiter, __decorate } from "tslib";
import { designTokens as alphaDefaultDesignTokens } from '@genesislcap/alpha-design-system';
import { loadRobotoFontFaces, designTokens as zeroDefaultDesignTokens, } from '@genesislcap/foundation-zero';
import { StandardLuminance, Combobox, Select } from '@microsoft/fast-components';
import { FASTElement, attr, customElement, observable, DOM } from '@microsoft/fast-element';
import { RadioGroup, Slider } from '@microsoft/fast-foundation';
import { fontFamilies } from '../../types';
import { DesignSystemEditorStyles as styles } from './design-system-editor.styles';
import { DesignSystemEditorTemplate as template } from './design-system-editor.template';
const mapDesignToken = ({ design_tokens: dsValues, }) => {
    return {
        accentColor: dsValues.color.accent.$value,
        neutralColor: dsValues.color.neutral.$value,
        luminance: dsValues.mode.luminance.$value,
        density: dsValues.style.density.$value,
        designUnit: dsValues.space.designUnit.$value,
        borderRadius: dsValues.style.borderRadius.$value,
        strokeWidth: dsValues.style.strokeWidth.$value,
        bodyFont: Object.keys(fontFamilies).find((font) => fontFamilies[font] === dsValues.fontFamily.bodyFont.$value),
        baseFontSize: dsValues.typography.baseFontSize.$value,
        baseLineHeight: dsValues.typography.baseLineHeight.$value,
    };
};
const ZeroDefaults = mapDesignToken(zeroDefaultDesignTokens);
const ZeroButtonApperance = {
    accent: 'primary-blue',
    neutral: 'neutral-grey',
    outline: 'primary-gradient',
};
const AlphaDefaults = mapDesignToken(alphaDefaultDesignTokens);
const AlphaButtonApperance = {
    accent: 'accent',
    neutral: 'neutral',
    outline: 'outline',
};
const DS = {
    zero: {
        prefix: 'zero',
        tokens: ZeroDefaults,
        buttons: ZeroButtonApperance,
    },
    alpha: {
        prefix: 'alpha',
        tokens: AlphaDefaults,
        buttons: AlphaButtonApperance,
    },
};
export var EventName;
(function (EventName) {
    EventName["STYLES_CHANGED"] = "styles-changed";
    EventName["DS_CHANGED"] = "design-system-changed";
    EventName["APPLY_CUSTOM_DESIGN"] = "apply-custom-design";
})(EventName || (EventName = {}));
let DesignSystemEditor = class DesignSystemEditor extends FASTElement {
    constructor() {
        super(...arguments);
        this.customDesignMode = false;
        this.ds = 'zero';
        this.darkMode = ZeroDefaults.luminance === StandardLuminance.DarkMode;
        this.luminance = ZeroDefaults.luminance;
        this.neutralColor = ZeroDefaults.neutralColor;
        this.accentColor = ZeroDefaults.accentColor;
        this.bodyFont = ZeroDefaults.bodyFont;
        this.density = ZeroDefaults.density;
        this.borderRadius = ZeroDefaults.borderRadius;
        this.strokeWidth = ZeroDefaults.strokeWidth;
        this.designUnit = ZeroDefaults.designUnit;
        this.baseFontSize = ZeroDefaults.baseFontSize;
        this.baseLineHeight = ZeroDefaults.baseLineHeight;
        this.changeHandler = (e) => {
            if (e.target instanceof RadioGroup ||
                e.target instanceof HTMLInputElement ||
                e.target instanceof Combobox ||
                e.target instanceof Select) {
                const { name, value } = e.target;
                if (name && value) {
                    this[name] = value;
                    this.styleUpdate();
                }
            }
            else if (e.target instanceof Slider) {
                const value = parseInt(e.target.value);
                const name = e.target.name;
                if (name && Number.isInteger(value)) {
                    this[name] = value;
                    this.styleUpdate();
                }
            }
        };
        this.modeChangeHandler = (e) => {
            this.darkMode = e.target.checked;
            this.luminance = this.darkMode ? StandardLuminance.DarkMode : StandardLuminance.LightMode;
            this.styleUpdate();
        };
        this.styleUpdate = () => {
            const detail = {
                accentColor: this.accentColor,
                neutralColor: this.neutralColor,
                bodyFont: fontFamilies[this.bodyFont],
                luminance: this.luminance,
                density: this.density,
                designUnit: this.designUnit,
                borderRadius: this.borderRadius,
                strokeWidth: this.strokeWidth,
                baseFontSize: this.baseFontSize,
                baseLineHeight: this.baseLineHeight,
            };
            this.$emit(EventName.STYLES_CHANGED, detail);
        };
        this.applyCustomDSTokens = (e) => {
            const detail = e.detail;
            if (detail) {
                DOM.queueUpdate(() => {
                    const customTokens = mapDesignToken(detail);
                    DS['custom'].tokens = customTokens;
                    this.setTokensFromDs(customTokens);
                    this.styleUpdate();
                });
            }
        };
        this.setTokensFromDs = (ds) => {
            this.neutralColor = ds.neutralColor;
            this.accentColor = ds.accentColor;
            this.bodyFont = ds.bodyFont;
            this.density = ds.density;
            this.designUnit = ds.designUnit;
            this.borderRadius = ds.borderRadius;
            this.strokeWidth = ds.strokeWidth;
            this.darkMode = ds.luminance === StandardLuminance.DarkMode;
            this.baseFontSize = ds.baseFontSize;
            this.baseLineHeight = ds.baseLineHeight;
        };
        this.setCustomTokens = () => {
            return fetch(`http://localhost:${location.port}/api/tokens`)
                .then((response) => response.json())
                .then((data) => {
                const customTokens = mapDesignToken(data);
                DS['custom'].tokens = customTokens;
            })
                .catch((error) => {
                console.error('Error fetching tokens:', error);
            });
        };
        this.resetChanges = () => __awaiter(this, void 0, void 0, function* () {
            if (this.ds === 'custom') {
                yield this.setCustomTokens();
            }
            this.setTokensFromDs(DS[this.ds].tokens);
            this.styleUpdate();
        });
    }
    customDesignModeChanged(oldValue, newValue) {
        const defaultDesignSystems = [
            { name: 'Zero', value: 'zero' },
            { name: 'Foundation UI', value: 'alpha' },
        ];
        const customDesignSystem = { name: 'Custom Design', value: 'custom' };
        if (this.customDesignMode) {
            DS['custom'] = Object.assign({}, DS.zero);
            this.designSystems = [customDesignSystem, ...defaultDesignSystems];
        }
        else {
            this.designSystems = defaultDesignSystems;
        }
    }
    dsChanged(oldValue, newValue) {
        const detail = {
            prefix: DS[this.ds].prefix,
            buttonAppearanceAccent: DS[this.ds].buttons.accent,
            buttonAppearanceNeutral: DS[this.ds].buttons.neutral,
            buttonAppearanceOutline: DS[this.ds].buttons.outline,
        };
        this.$emit(EventName.DS_CHANGED, detail);
    }
    connectedCallback() {
        super.connectedCallback();
        loadRobotoFontFaces();
        DOM.queueUpdate(() => {
            this.styleUpdate();
        });
        document.body.addEventListener(EventName.APPLY_CUSTOM_DESIGN, (e) => {
            this.applyCustomDSTokens(e);
        });
    }
};
__decorate([
    attr({ mode: 'boolean', attribute: 'custom-design-mode' })
], DesignSystemEditor.prototype, "customDesignMode", void 0);
__decorate([
    observable
], DesignSystemEditor.prototype, "designSystems", void 0);
__decorate([
    observable
], DesignSystemEditor.prototype, "ds", void 0);
__decorate([
    attr
], DesignSystemEditor.prototype, "darkMode", void 0);
__decorate([
    observable
], DesignSystemEditor.prototype, "neutralColor", void 0);
__decorate([
    observable
], DesignSystemEditor.prototype, "accentColor", void 0);
__decorate([
    observable
], DesignSystemEditor.prototype, "bodyFont", void 0);
__decorate([
    observable
], DesignSystemEditor.prototype, "density", void 0);
__decorate([
    observable
], DesignSystemEditor.prototype, "borderRadius", void 0);
__decorate([
    observable
], DesignSystemEditor.prototype, "strokeWidth", void 0);
__decorate([
    observable
], DesignSystemEditor.prototype, "designUnit", void 0);
__decorate([
    observable
], DesignSystemEditor.prototype, "baseFontSize", void 0);
__decorate([
    observable
], DesignSystemEditor.prototype, "baseLineHeight", void 0);
DesignSystemEditor = __decorate([
    customElement({
        name: 'design-system-editor',
        template,
        styles,
    })
], DesignSystemEditor);
export { DesignSystemEditor };
